from datetime import date as Date
from datetime import datetime as DateTime
from datetime import timedelta as TimeDelta
import re

today = Date.today

def absolute_date(info):
    if isinstance(info, Date):
        return info
    elif isinstance(info, tuple):
        return Date(*info)
    elif isinstance(info, str):
        return DateTime.strptime(info, '%Y-%m-%d').date()
    else:
        raise RuntimeError("Invalid date %s" % info)

def date_for(time_info):
    if isinstance(time_info, str):
        if time_info == 'tomorrow': return today() + TimeDelta(1)
        if time_info == 'today': return today()
        days_match = re.match('^(\d+)\s+days?$', time_info)
        if days_match:
            days = int(days_match.group(1))
            return today() + TimeDelta(days)
        d_match = re.match('^(\d+)d$', time_info)
        if d_match:
            days = int(d_match.group(1))
            return today() + TimeDelta(days)
    return absolute_date(time_info)



from unittest import TestCase
from tempfile import TemporaryDirectory
from pathlib import Path
from unittest import mock
from io import StringIO
from datetime import date as Date

from busy.task import Task
from busy.commander import Commander
from busy.system import System

class TestCommandDefer(TestCase):

    def test_defer(self):
        with TemporaryDirectory() as t:
            p = Path(t, 'todo.txt')
            p.write_text('a\nb\nc\nd')
            c = Commander(root=t)
            c.handle('defer','2','--to','2019-09-06')
            o = p.read_text()
            self.assertEqual(o, 'a\nc\nd\n')
            o2 = Path(t, 'plan.txt').read_text()
            self.assertEqual(o2, '2019-09-06|b\n')

    def test_defer_for(self):
        with TemporaryDirectory() as t:
            p = Path(t, 'todo.txt')
            p.write_text('a\nb\nc\nd')
            c = Commander(root=t)
            c.handle('defer','2','--for','2019-09-06')
            o = p.read_text()
            self.assertEqual(o, 'a\nc\nd\n')
            o2 = Path(t, 'plan.txt').read_text()
            self.assertEqual(o2, '2019-09-06|b\n')

    def test_defer_days(self):
        with TemporaryDirectory() as t:
            p = Path(t, 'todo.txt')
            p.write_text('a\nb\nc\nd')
            with mock.patch('busy.future.today', lambda : Date(2019,2,11)):
                c = Commander(root=t)
                c.handle('defer','2','--for','1 day')
                o = p.read_text()
                self.assertEqual(o, 'a\nc\nd\n')
                o2 = Path(t, 'plan.txt').read_text()
                self.assertEqual(o2, '2019-02-12|b\n')

    def test_defer_d(self):
        with TemporaryDirectory() as t:
            p = Path(t, 'todo.txt')
            p.write_text('a\nb\nc\nd')
            with mock.patch('busy.future.today', lambda : Date(2019,2,11)):
                c = Commander(root=t)
                c.handle('defer','2','--for','5d')
                o = p.read_text()
                self.assertEqual(o, 'a\nc\nd\n')
                o2 = Path(t, 'plan.txt').read_text()
                self.assertEqual(o2, '2019-02-16|b\n')

    def test_defer_with_input(self):
        with TemporaryDirectory() as t:
            p = Path(t, 'todo.txt')
            p.write_text('a\nb\n')
            c = Commander(root=t)
            with mock.patch('sys.stdin', StringIO('2019-08-24')):
                c.handle('defer')
                o = p.read_text()
                self.assertEqual(o, 'b\n')
                o2 = Path(t, 'plan.txt').read_text()
                self.assertEqual(o2, '2019-08-24|a\n')

    def test_defer_without_date_raises_error(self):
        with TemporaryDirectory() as t:
            p = Path(t, 'todo.txt')
            p.write_text('a\nb\n')
            c = Commander(root=t)
            with mock.patch('sys.stdin', None):
                with self.assertRaises(RuntimeError):
                    c.handle('defer')



from .queue import TodoQueue
from .queue import PlanQueue
from .task import Task

import busy.future
import busy

class System:

    def __init__(self, *items, todos=None, plans=None):
        self.todos = todos if todos else TodoQueue()
        self.plans = plans if plans else PlanQueue()
        self.add(*items)

    def add(self, *items):
        self.todos.add(*items)

    def pop(self, *criteria):
        self.todos.pop(*criteria)

    def drop(self, *criteria):
        self.todos.drop(*criteria)

    def defer(self, date, *criteria):
        indices = self.todos.select(*(criteria or [1]))
        plans = [self.todos.get(i+1).as_plan(date) for i in indices]
        self.plans.add(*plans)
        self.todos.delete_by_indices(*indices)

    def activate(self, *criteria, today=False):
        if today:
            func = lambda t: t.plan_date <= busy.future.today()
            indices = self.plans.select(func)
        else:
            indices = self.plans.select(*criteria)
        tasks = [self.plans.get(i+1).as_todo() for i in indices]
        self.todos.add(*tasks)
        self.plans.delete_by_indices(*indices)

    def manage(self, *criteria):
        tasklist = self.todos.list(*criteria)
        indices = [i[0]-1 for i in tasklist]
        before = ''.join([str(i[1])+'\n' for i in tasklist])
        after = busy.editor(before).split('\n')
        new_tasks = [Task(i) for i in after if i]
        self.todos.replace(indices, new_tasks)
        # self.todos.delete_by_indices(*indices)
        # self.add(*new_tasks)



from unittest import TestCase
from pathlib import Path
from tempfile import TemporaryDirectory

from busy.file import TodoFile
from busy.file import PlanFile
from busy.queue import Queue
from busy.task import Task

class TestFile(TestCase):

    def test_load_tasks(self):
        with TemporaryDirectory() as d:
            p = Path(d) / 'todo.txt'
            p.write_text('a\nb\n')
            f = TodoFile(Path(d))
            q = f.queue
            self.assertIsInstance(q, Queue)
            self.assertEqual(str(q.get()), 'a')
            self.assertIsInstance(q.get(), Task)

    def test_load_if_not_there(self):
        with TemporaryDirectory() as d:
            f = TodoFile(Path(d))
            q = f.queue
            self.assertIsInstance(q, Queue)
            self.assertEqual(q.count(), 0)

    def test_save_tasks(self):
        with TemporaryDirectory() as d:
            p = Path(d) / 'todo.txt'
            f1 = TodoFile(Path(d))
            q1 = f1.queue
            q1.add(Task('a'))
            f1.save()
            f2 = TodoFile(Path(d))
            q2 = f2.queue
            self.assertEqual(str(q2.get()), 'a')

    def test_plan_file(self):
        with TemporaryDirectory() as d:
            p = Path(d)
            f = PlanFile(p)
            t = Task('a').as_plan((2019,2,3))
            f.queue.add(t)
            f.save()
            q2 = PlanFile(p).queue
            self.assertEqual(q2.get().plan_date.day, 3)

    def test_plan_file_format(self):
        with TemporaryDirectory() as d:
            p = Path(d) / 'plan.txt'
            p.write_text('2018-12-01|a\n2018-12-09|b')
            q = PlanFile(Path(d)).queue
            self.assertEqual(q.get().plan_date.month, 12)



from unittest import TestCase
from tempfile import TemporaryDirectory
from pathlib import Path
from unittest import mock

from busy.system import System
from busy.root import Root
from busy.file import File

class TestRoot(TestCase):

    def test_root(self):
        with TemporaryDirectory() as d:
            sd = Root(Path(d))
            s = sd.system
            self.assertIsInstance(s, System)

    def test_add_todo(self):
        with TemporaryDirectory() as td:
            sd1 = Root(Path(td))
            sd1.system.add('a')
            sd1.save()
            sd2 = Root(Path(td))
            self.assertEqual(str(sd2.system.todos.get()),'a')

    def test_make_dir_pater(self):
        r = Root()
        with TemporaryDirectory() as td:
            r.path = Path(td)
            r.system.add('a')
            r.save()
            r2 = Root(Path(td))
            self.assertEqual(str(r2.system.todos.get()),'a')

    def test_env_var_as_backup(self):
        with TemporaryDirectory() as td:
            with mock.patch.dict('os.environ', {'BUSY_ROOT': td}):
                sd1 = Root()
                sd1.system.add('a')
                sd1.save()
                f = Path(td) / 'todo.txt'
                self.assertEqual(f.read_text(),'a\n')

    def test_user_root(self):
        with TemporaryDirectory() as td:
            with mock.patch.dict('os.environ', clear=True):
                with mock.patch('pathlib.Path.home', lambda : Path(td)):
                    sd1 = Root()
                    sd1.system.add('a')
                    sd1.save()
                    f = Path(td) / '.busy' / 'todo.txt'
                    self.assertEqual(f.read_text(),'a\n')



from pathlib import Path
from tempfile import TemporaryDirectory
import os

from .system import System
from .task import Task
from .file import TodoFile
from .file import PlanFile

class Root:

    def __init__(self, path=None):
        if path: self.path = path

    @property
    def path(self):
        if not hasattr(self, '_path'):
            env_var = os.environ.get('BUSY_ROOT')
            self._path = Path(env_var if env_var else Path.home() / '.busy')
            if not self._path.is_dir(): self._path.mkdir()
        return self._path

    @path.setter
    def path(self, value):
        assert not hasattr(self, '_path')
        path = Path(value) if isinstance(value, str) else value
        assert isinstance(path, Path) and path.is_dir()
        self._path = path

    @property
    def system(self):
        if not hasattr(self, '_system'):
            self._todo_file = TodoFile(self.path)
            self._plan_file = PlanFile(self.path)
            todos = self._todo_file.queue
            plans = self._plan_file.queue
            self._system = System(todos=todos, plans=plans)
        return self._system

    def save(self):
        self._todo_file.save()
        self._plan_file.save()



from unittest import TestCase
import datetime

from busy.task import Task


class TestTask(TestCase):

    def test_task(self):
        t = Task('b')
        self.assertIsInstance(t, Task)

    def test_description_as_string(self):
        t = Task('a')
        self.assertEqual(t.description, 'a')

    def test_task_requires_description(self):
        with self.assertRaises(AssertionError):
            t = Task('')

    def test_plan(self):
        t = Task('a')
        d = datetime.date(2018,12,1)
        t.as_plan(d)

    def test_plan_as_tuple(self):
        t = Task('b')
        t.as_plan((2018,12,1))
        self.assertEqual(t.plan_date.month, 12)

    def test_plan_requires_date(self):
        t = Task('c')
        with self.assertRaises(RuntimeError):
            t.as_plan(54)

    def test_plan_with_date_as_string(self):
        t = Task ('d')
        t.as_plan('2019-04-05')
        self.assertEqual(t.plan_date.day, 5)

    def test_create_task_with_dict(self):
        t = Task(plan_date=(2019,4,15), description='a')
        self.assertEqual(t.plan_date.month, 4)

    def test_tags(self):
        t = Task('f #a')
        self.assertEqual(t.tags, ['a'])

    def test_project(self):
        t = Task("k #oNe #two")
        self.assertEqual(t.project, 'one')



# Method calls on queue use indices that start at 1

from .selector import Selector
from .task import Task

class Queue:

    def __init__(self, *items):
        self._items = []
        self.add(*items)

    def all(self):
        return self._items

    def count(self):
        return len(self._items)

    def add(self, *items):
        for item in items:
            self._items.append(Task.create(item))

    def get(self, index=1):
        return self._items[index-1] if self._items else None

    def select(self, *criteria):
        selector = Selector(criteria)
        return selector.indices(self._items)

    def _split(self, *criteria):
        return self._split_by_indices(*self.select(*criteria))

    def _split_by_indices(self, *indices):
        inlist = [t for i,t in enumerate(self._items) if i in indices]
        outlist = [t for i,t in enumerate(self._items) if i not in indices]
        return (inlist, outlist)

    def pop(self, *criteria):
        hilist, lolist = self._split(*criteria or [len(self._items)])
        self._items = hilist + lolist
        return hilist

    def drop(self, *criteria):
        lolist, hilist = self._split(*criteria or [1])
        self._items = hilist + lolist
        return lolist

    def delete(self, *criteria):
        killlist, keeplist = self._split(*criteria)
        self._items = keeplist
        return killlist

    def delete_by_indices(self, *indices):
        killlist, keeplist = self._split_by_indices(*indices)
        self._items = keeplist
        return killlist

    def list(self, *criteria):
        return [(i+1, self._items[i]) for i in self.select(*criteria)]

    def replace(self, indices, newvalues):
        while newvalues and indices:
            self._items[indices.pop(0)] = newvalues.pop(0)
        while indices:
            del self._items[indices.pop()]
        self._items.extend(newvalues)

    @property
    def strings(self):
        return [str(i) for i in self._items]

class TodoQueue(Queue):
    schema = ['description']
    listfmt = "{1.description}"

class PlanQueue(Queue):
    schema = ['plan_date', 'description']
    listfmt = "{1.plan_date:%Y-%m-%d}  {1.description}"



from csv import DictReader
from csv import DictWriter

from .queue import TodoQueue, PlanQueue

class File:

    def __init__(self, dirpath):
        self._path = dirpath / self.filename
        if self._path.is_file():
            with open(self._path) as datafile:
                reader = DictReader(datafile, self.schema, delimiter="|")
                self.queue = self.queueclass(*reader)

    @property
    def queue(self):
        if not hasattr(self, '_queue'): self._queue = self.queueclass()
        return self._queue

    @queue.setter
    def queue(self, value):
        assert isinstance(value, self.queueclass)
        self._queue = value

    @property
    def schema(self):
        return self.queueclass.schema

    def save(self):
        with open(self._path, 'w') as datafile:
            writer = DictWriter(datafile, self.schema, delimiter="|")
            for item in self.queue.all():
                values = dict([(f, getattr(item,f)) for f in self.schema])
                writer.writerow(values)

class TodoFile(File):
    queueclass = TodoQueue
    filename = 'todo.txt'

class PlanFile(File):
    queueclass = PlanQueue
    filename = 'plan.txt'



from unittest import TestCase
from tempfile import TemporaryDirectory
from pathlib import Path
from unittest import mock
from io import StringIO
from datetime import date as Date

from busy.task import Task
from busy.commander import Commander
from busy.system import System

class TestCommandDrop(TestCase):

    def test_drop(self):
        with TemporaryDirectory() as t:
            p = Path(t, 'todo.txt')
            p.write_text('a\nb\nc\nd')
            c = Commander(root=t)
            c.handle('drop','2','4')
            o = p.read_text()
            self.assertEqual(o, 'a\nc\nb\nd\n')

    def test_pop(self):
        with TemporaryDirectory() as t:
            p = Path(t, 'todo.txt')
            p.write_text('a\nb\nc\nd')
            c = Commander(root=t)
            c.handle('pop','2','4')
            o = p.read_text()
            self.assertEqual(o, 'b\nd\na\nc\n')



from datetime import date as Date
from datetime import datetime as DateTime
from datetime import timedelta as TimeDelta
import re

from .future import absolute_date

TODO_STATE = 't'
PLAN_STATE = 'p'
DONE_STATE = 'd'

class Task:

    def __init__(self, description=None, plan_date=None):
        assert isinstance(description, str)
        assert description
        self._description = description
        self._state = TODO_STATE
        if plan_date: self.as_plan(plan_date)

    def __str__(self):
        return self._description

    @property
    def description(self):
        return self._description

    def as_plan(self, date):
        self._state = PLAN_STATE
        self._plan_date = absolute_date(date)
        return self

    def as_todo(self):
        self._state = TODO_STATE
        return self

    @property
    def plan_date(self):
        return self._plan_date

    @classmethod
    def create(self, value):
        if isinstance(value, self):
            return value
        elif isinstance(value, str):
            return self(value)
        elif isinstance(value, dict):
            return self(**value)

    @property
    def tags(self):
        words = self.description.split()
        return [w[1:].lower() for w in words if w.startswith('#')]

    @property
    def project(self):
        tags = self.tags
        return tags[0] if tags else None



import sys

from .commander import Commander

def main():
    try:
        output = Commander().handle(*sys.argv[1:])
    except RuntimeError as error:
        output = f"Error: {str(error)}"
    if output: print(output)

if __name__ == '__main__': main()



import os
from setuptools import setup
from setuptools import find_packages
from pathlib import Path

with open(os.path.join(os.path.dirname(__file__),'version')) as versionfile:
    version = versionfile.read().strip()

long_description = (Path(__file__).parent / 'README.md').read_text()

setup(name='busy',
    version=version,
    description='Command-line task and plan management tool',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='http://gitlab.com/francispotter/busy',
    author='Francis Potter',
    author_email='busy@fpotter.com',
    license='MIT',
    packages=find_packages(),
    entry_points={'console_scripts':['busy=busy.__main__:main']},
    zip_safe=False)



from unittest import TestCase
import datetime

from busy.system import System
from busy.task import Task

class TestSystem(TestCase):

    def test_get(self):
        q = System()
        q.add(Task('a'))
        t = q.todos.get()
        self.assertEqual(str(t),'a')

    def test_list(self):
        s = System()
        s.add(Task('a'))
        s.add(Task('b'))
        i = s.todos.list()
        self.assertEqual(len(i), 2)
        self.assertEqual(i[1][0], 2)
        self.assertEqual(str(i[0][1]), 'a')
        self.assertIsInstance(i[1][1], Task)

    def test_defer(self):
        s = System()
        s.add(Task('a'))
        d = datetime.date(2018,12,25)
        s.defer(d)
        self.assertEqual(s.plans.count(), 1)
        self.assertEqual(s.plans.get(1).plan_date.year, 2018)

    def test_pop(self):
        s = System()
        t1 = Task('a')
        t2 = Task('b')
        s.add(t1)
        s.add(t2)
        s.pop()
        i = s.todos.list()
        self.assertEqual(len(i), 2)
        self.assertEqual(str(i[0][1]), 'b')

    def test_by_number(self):
        s = System()
        s.add(Task('a'))
        s.add(Task('b'))
        t = s.todos.get(2)
        self.assertEqual(str(t), 'b')

    def test_create_with_string(self):
        s = System()
        s.add('a')
        self.assertEqual(str(s.todos.get()), 'a')
        self.assertIsInstance(s.todos.get(), Task)

    def test_create_with_multiple_strings(self):
        s = System('a','b','c')
        self.assertIsInstance(s.todos.get(), Task)
        self.assertEqual(s.todos.count(), 3)
        self.assertEqual(str(s.todos.get(2)), 'b')

    def test_select_multiple(self):
        s = System('a','b','c')
        t = s.todos.select(1,3)
        self.assertEqual(len(t), 2)

    def test_list_plans(self):
        s = System('a','b')
        s.defer((2018,12,4))
        self.assertEqual(s.plans.count(), 1)

    def test_defer_by_index(self):
        s = System('a','b')
        s.defer((2018,12,4),2)
        t = s.todos.get()
        self.assertEqual(s.plans.count(),1)
        self.assertEqual(str(t), 'a')

    def test_defer_multiple(self):
        s = System('a','b','c')
        s.defer((2018,12,5),1,3)
        p = s.plans.get(2)
        self.assertEqual(str(p), 'c')

    def test_list_by_criteria(self):
        s = System('a','b','c')
        i = s.todos.list(2,3)
        self.assertEqual(len(i), 2)
        self.assertEqual(str(i[1][1]), 'c')

    def test_drop(self):
        s = System('a','b','c')
        s.drop()
        self.assertEqual(str(s.todos.get()), 'b')
        self.assertEqual(str(s.todos.get(3)), 'a')

    def test_drop_by_criteria(self):
        s = System('a','b','c','d')
        s.drop('2-3')
        self.assertEqual(str(s.todos.get(1)), 'a')
        self.assertEqual(str(s.todos.get(2)), 'd')
        self.assertEqual(str(s.todos.get(3)), 'b')
        self.assertEqual(str(s.todos.get(4)), 'c')



import subprocess
from tempfile import NamedTemporaryFile
from pathlib import Path

PYTHON_VERSION = (3,6,5)

def editor(arg):
    with NamedTemporaryFile() as tempfile:
        Path(tempfile.name).write_text(arg)
        subprocess.run(['sensible-editor', tempfile.name])
        return Path(tempfile.name).read_text()



from argparse import ArgumentParser
from tempfile import TemporaryDirectory
import sys
from tempfile import TemporaryFile
from pathlib import Path

from .root import Root
from . import PYTHON_VERSION
import busy
from .task import Task
from .future import date_for

class Commander:

    def __init__(self, *args, root=None):
        if sys.version_info < PYTHON_VERSION:
            message = ("Busy requires Python version %i.%i.%i or higher" %
                PYTHON_VERSION)
            raise RuntimeError(message)
        if root: self.root = Root(root)

    def handle(self, *args):
        parsed, remaining = self._parser.parse_known_args(args)
        parsed.criteria = remaining
        if parsed.root: self.root = Root(parsed.root)
        if hasattr(parsed, 'command'):
            command = parsed.command(self.root)
            return command.execute(parsed)

    @property
    def root(self):
        if not hasattr(self, '_root'): self._root = Root()
        return self._root

    @root.setter
    def root(self, value):
        assert not hasattr(self, '_path')
        assert isinstance(value, Root)
        self._root = value

    @classmethod
    def register(self, command_class):
        if not hasattr(self, '_parser'):
            self._parser = ArgumentParser()
            self._parser.add_argument('--root', action='store')
            self._subparsers = self._parser.add_subparsers()
        subparser = self._subparsers.add_parser(command_class.command)
        subparser.set_defaults(command=command_class)
        command_class.register(subparser)


class Command:

    def __init__(self, root):
        self._root = root

    @property
    def _system(self):
        return self._root.system

    @classmethod
    def register(self, parser):
        pass

    def execute(self, parsed):
        method = getattr(self._system, self.command)
        result = method(*parsed.criteria)
        self._root.save()
        return result

    def _list(self, queue, tasklist):
        fmtstring = "{0:>6}  " + queue.listfmt
        texts = [fmtstring.format(i, t) for i,t in tasklist]
        return '\n'.join(texts)


class ListCommand(Command):

    command = 'list'

    @classmethod
    def register(self, parser):
        parser.add_argument('--plans', action='store_true')

    def execute(self, parsed):
        queue = self._system.plans if parsed.plans else self._system.todos
        tasklist = queue.list(*parsed.criteria)
        return self._list(queue, tasklist)

Commander.register(ListCommand)


class AddCommand(Command):

    command = 'add'

    @classmethod
    def register(self, parser):
        parser.add_argument('--task')

    def execute(self, parsed):
        if hasattr(parsed, 'task') and parsed.task:
            task = parsed.task
        else:
            task = input('Task: ')
        self._root.system.add(task)
        self._root.save()

Commander.register(AddCommand)


class DropCommand(Command): command = 'drop'
Commander.register(DropCommand)


class PopCommand(Command): command = 'pop'
Commander.register(PopCommand)


class DeleteCommand(Command):

    command = 'delete'

    @classmethod
    def register(self, parser):
        parser.add_argument('--yes', action='store_true')

    def execute(self, parsed):
        tasklist = self._system.todos.list(*parsed.criteria or [1])
        indices = [i[0]-1 for i in tasklist]
        if hasattr(parsed, 'yes') and parsed.yes:
            confirmed = True
        else:
            print('\n'.join([str(i[1]) for i in tasklist]))
            confirmed = input('Delete? (Y/n) ').startswith('Y')
        if not confirmed:
            print("Deletion must be confirmed")
        else:
            self._system.todos.delete_by_indices(*indices)
            self._root.save()

Commander.register(DeleteCommand)


class GetCommand(Command):

    command = 'get'

    def execute(self, parsed):
        if parsed.criteria:
            message = ("The `get` command only returns the active task - "
                "repeat without criteria")
            raise RuntimeError(message)
        else:
            return str(self._system.todos.get() or '')

Commander.register(GetCommand)


class DeferCommand(Command):

    command = 'defer'

    @classmethod
    def register(self, parser):
        parser.add_argument('--to','--for',dest='time_info')

    def execute(self, parsed):
        tasklist = self._system.todos.list(*parsed.criteria or [1])
        indices = [i[0]-1 for i in tasklist]
        if hasattr(parsed, 'time_info') and parsed.time_info:
            time_info = parsed.time_info
        else:
            print('\n'.join([str(i[1]) for i in tasklist]))
            time_info = input('Defer to: ')
        self._root.system.defer(date_for(time_info), *parsed.criteria)
        self._root.save()

Commander.register(DeferCommand)


class ActivateCommand(Command):

    command = 'activate'

    @classmethod
    def register(self, parser):
        parser.add_argument('--today','-t', action='store_true')

    def execute(self, parsed):
        if hasattr(parsed, 'today') and parsed.today:
            self._system.activate(today=True)
        else:
            self._system.activate(*parsed.criteria)
        self._root.save()

Commander.register(ActivateCommand)


class StartCommand(Command):

    command = 'start'

    @classmethod
    def register(self, parser):
        parser.add_argument('project', action='store', nargs='?')

    def execute(self, parsed):
        if parsed.criteria:
            raise RuntimeError('Start takes only an optional project name')
        self._system.activate(today=True)
        queue = self._system.todos
        if queue.count() < 1:
            raise RuntimeError('There are no active tasks')
        project = parsed.project or queue.get().project
        if not project:
            raise RuntimeError('The `start` command required a project')
        self._system.manage(project)
        result = queue.pop(project)
        self._root.save()


Commander.register(StartCommand)


class ManageCommand(Command):

    command = 'manage'

    def execute(self, parsed):
        self._system.manage(*parsed.criteria)
        self._root.save()

Commander.register(ManageCommand)
